/** 
 * XMLSec library
 *
 * "XML Encryption" implementation
 *  http://www.w3.org/TR/xmlenc-core
 * 
 * See Copyright for the status of this software.
 * 
 * Author: Aleksey Sanin <aleksey@aleksey.com>
 */
#ifndef __XMLSEC_XMLENC_H__
#define __XMLSEC_XMLENC_H__    

#ifndef XMLSEC_NO_XMLENC

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */ 
#include <stdio.h>

#include <libxml/tree.h>
#include <libxml/parser.h> 

#include <xmlsec/xmlsec.h>
#include <xmlsec/keys.h>
#include <xmlsec/transforms.h>


typedef struct _xmlEncCtx *xmlEncCtxPtr; 
typedef struct _xmlEncResult *xmlEncResultPtr; 

extern const xmlChar xmlEncTypeElement[]; /* "http://www.w3.org/2001/04/xmlenc#Element"; */
extern const xmlChar xmlEncTypeContent[]; /* "http://www.w3.org/2001/04/xmlenc#Content"; */


/** 
 * XML Encrypiton context
 */
typedef struct _xmlEncCtx {
    /* keys search context */  
    xmlSecKeyPtr		key;
    xmlSecTransformId		encryptionMethod;  
    void			*encryptionData;
    const xmlSecKeysReadContext	*keysReadContext;    
    
    /* flags */
    int				ignoreType;
} xmlEncCtx;

/**
 * XML Encrypiton results
 */
typedef struct _xmlEncResult {
    xmlEncCtxPtr		ctx;
    xmlNodePtr			self;
    int				encrypt;
    xmlChar			*id;
    xmlChar			*type;
    xmlChar			*mimeType;
    xmlChar			*encoding;    
    xmlSecTransformId		encryptionMethod;
    xmlSecKeyPtr		key;            
    xmlBufferPtr		buffer;
    int				replaced;
} xmlEncResult;

typedef xmlNodePtr		xmlEncTmplPtr;

/**
 * XML Encrypiton context methods
 */
xmlEncCtxPtr		xmlEncCtxCreate			(const xmlSecKeysReadContext *keysReadContext);
void 			xmlEncCtxDestroy		(xmlEncCtxPtr ctx);


/**
 * Encryption
 */
int			xmlEncEncryptMemory		(xmlEncCtxPtr ctx,
							 xmlEncTmplPtr tmpl,
							 const unsigned char *buf,
							 size_t size,
							 xmlEncResultPtr *result);
int			xmlEncEncryptUri		(xmlEncCtxPtr ctx,
							 xmlEncTmplPtr tmpl,
							 const char *uri,
							 xmlEncResultPtr *result);
int			xmlEncEncryptXmlNode		(xmlEncCtxPtr ctx,
							 xmlEncTmplPtr tmpl,
							 xmlNodePtr src,
							 xmlEncResultPtr *result);

/**
 * Decryption
 */
int			xmlEncDecrypt			(xmlEncCtxPtr ctx,
							 xmlNodePtr encDataNode,
							 xmlEncResultPtr *result);
 
				
/**
 * XML Enc Result
 */		
xmlEncResultPtr		xmlEncResultCreate		(xmlEncCtxPtr ctx,
							 int encrypt,
							 xmlNodePtr node);
void 			xmlEncResultDestroy		(xmlEncResultPtr result);
void			xmlEncResultDebugDump		(FILE *output,
							 xmlEncResultPtr result);

/** 
 * Encryption Template
 */
xmlEncTmplPtr		xmlEncTmplCreate		(const xmlChar *id,
							 const xmlChar *type,
							 const xmlChar *mimeType,
							 const xmlChar *encoding);
void 			xmlEncTmplDestroy		(xmlEncTmplPtr tmpl);
xmlNodePtr		xmlEncTmplAddEncMethod		(xmlEncTmplPtr tmpl,
							 xmlSecTransformId encMethod);
xmlNodePtr		xmlEncTmplAddKeyInfo		(xmlEncTmplPtr tmpl);							 
xmlNodePtr		xmlEncTmplAddEncProperties	(xmlEncTmplPtr tmpl,
							 const xmlChar *id); 
xmlNodePtr		xmlEncTmplAddEncProperty	(xmlEncTmplPtr tmpl,
							 const xmlChar *id,
							 const xmlChar *target);
xmlNodePtr		xmlEncTmplAddCipherValue	(xmlEncTmplPtr tmpl);
xmlNodePtr		xmlEncTmplAddCipherRef		(xmlEncTmplPtr tmpl,
							 const xmlChar *uri);
xmlNodePtr		xmlEncTmplAddCipherRefTransform	(xmlEncTmplPtr tmpl,
							 xmlSecTransformId transform);
#define 		xmlEncTmplGetEncData(tmpl)	\
				((xmlNodePtr)(tmpl))							 

xmlEncTmplPtr		xmlKeyInfoAddEncryptedKey	(xmlNodePtr keyInfoNode,
							 const xmlChar *id,
							 const xmlChar *type,
							 const xmlChar *recipient);

/**
 * EncryptedKey node
 */
xmlSecKeyPtr	 	xmlEncryptedKeyNodeRead		(xmlNodePtr cur, 
							 xmlSecTransformId id, 
							 xmlSecKeyType type, 
							 const xmlSecKeysReadContext *ctx);
int		 	xmlEncryptedKeyNodeWrite	(xmlNodePtr cur, 
							 xmlSecKeyPtr key, 
							 xmlSecKeyType type,
							 const xmlSecKeysReadContext *ctx);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* XMLSEC_NO_XMLENC */

#endif /* __XMLSEC_XMLENC_H__ */

